package com.example.ellegadodepintia.exploradoresDePintia

import android.animation.AnimatorSet
import android.animation.ObjectAnimator
import android.app.Activity
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import android.view.animation.AccelerateDecelerateInterpolator
import android.widget.Button
import android.widget.ImageView
import android.widget.ProgressBar
import android.widget.TextView
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.example.ellegadodepintia.MapActivity
import com.example.ellegadodepintia.R
import com.example.ellegadodepintia.exploradoresDePintia.model.Atributo
import com.example.ellegadodepintia.exploradoresDePintia.model.GameState
import com.example.ellegadodepintia.exploradoresDePintia.model.objeto.ObjetoDeValor
import com.example.ellegadodepintia.repositorios.RepositorioJugador
import com.example.ellegadodepintia.ruleta.Ruleta

class Resumen : AppCompatActivity() {
    private lateinit var startForResult: ActivityResultLauncher<Intent>
    private var gananciaTotal = 0

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_resumen_partida)

        startForResult = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                val mensajeResultado = result.data?.getStringExtra("resultado")!!.toInt()
                println(mensajeResultado)
                val textoResumen = findViewById<TextView>(R.id.textoResumen)
                "Has usado el ticker para generar aun más monedas. Enhorabuena, has conseguido $mensajeResultado monedas.".also {
                    textoResumen.text = it
                }
                GameState.jugador.monedas+=mensajeResultado-gananciaTotal
                GameState.jugador.monedas.toString()
                    .also { findViewById<TextView>(R.id.moneyText).text = it }
                RepositorioJugador.setMonedas(GameState.jugador.monedas)
                findViewById<Button>(R.id.botonRuleta).visibility = View.GONE

            }
        }


        if (GameState.jugador.atributos[Atributo.Vida]!! <= 0) {
            mostrarDerrota()
        } else {
            calcularGanancias()
        }
        GameState.jugador.monedas.toString()
            .also { findViewById<TextView>(R.id.moneyText).text = it }
        val continuarButton = findViewById<Button>(R.id.botonContinuar)
        continuarButton.setOnClickListener {
            val intent = Intent(this, MapActivity::class.java)
            startActivity(intent)
            GameState.reset()
        }

        val barraVida = findViewById<ProgressBar>(R.id.barraVida)
        barraVida.max = GameState.jugador.vidaMax
        barraVida.progress = GameState.jugador.atributos[Atributo.Vida]!!
    }

    private fun mostrarDerrota() {
        val textoResumen = findViewById<TextView>(R.id.textoResumen)
        val imagenProtagonista = findViewById<ImageView>(R.id.imagenProtagonista)

        "El investigador ha sido derrotado, ha perdido todos sus objetos del inventario".also {
            textoResumen?.text = it
        }
        imagenProtagonista?.setImageResource(R.drawable.protagonista_die_06)

    }

    private fun calcularGanancias() {
        val imagenProtagonista = findViewById<ImageView>(R.id.imagenProtagonista)
        val textoResumen = findViewById<TextView>(R.id.textoResumen)
        val recyclerView: RecyclerView = findViewById(R.id.resumenRecyclerView)
        recyclerView.layoutManager = LinearLayoutManager(this)
        var nObjetosValor = 0
        textoResumen?.text = ""

        val movimientoLateral =
            ObjectAnimator.ofFloat(imagenProtagonista, "translationX", -100f, 100f)
        movimientoLateral.duration = 1500
        movimientoLateral.repeatCount = ObjectAnimator.INFINITE
        movimientoLateral.repeatMode = ObjectAnimator.REVERSE

        val reboteVertical = ObjectAnimator.ofFloat(imagenProtagonista, "translationY", 0f, 30f, 0f)
        reboteVertical.duration = 600
        reboteVertical.repeatCount = ObjectAnimator.INFINITE
        reboteVertical.repeatMode = ObjectAnimator.REVERSE
        reboteVertical.interpolator = AccelerateDecelerateInterpolator()

        val animatorSet = AnimatorSet()
        animatorSet.playTogether(movimientoLateral, reboteVertical)
        animatorSet.start()

        val objetosDeValor: MutableList<ObjetoDeValor> = mutableListOf()
        for (objeto in GameState.jugador.inventario) {
            if (objeto is ObjetoDeValor) {
                nObjetosValor++
                gananciaTotal += objeto.coste
                objetosDeValor.add(objeto)
                objeto.usar()
            }
        }
        println(objetosDeValor.isEmpty())
        recyclerView.adapter = ResumenAdapter(objetosDeValor)


        if (nObjetosValor == 0) {
            "El investigador no ha conseguido ningún objeto de valor".also {
                textoResumen?.text = it
            }
        } else {
            "${textoResumen.text} Enhorabuena, has conseguido $gananciaTotal monedas.".also {
                textoResumen.text = it
            }
            val botonRuleta = findViewById<Button>(R.id.botonRuleta)
            val tieneTicket = GameState.jugador.inventario.any { it.nombre == "Ticket" }
            println(tieneTicket)
            if (!tieneTicket) {
                (botonRuleta.parent as? ViewGroup)?.removeView(botonRuleta)
            } else {
                botonRuleta.visibility = View.VISIBLE
                botonRuleta.setOnClickListener {
                    val intent = Intent(this, Ruleta::class.java)
                    intent.putExtra("PUNTUACION_ACTUAL", gananciaTotal)
                    startForResult.launch(intent)
                }
            }
        }
    }
    override fun onBackPressed() {
        val flag = intent.getBooleanExtra("FLAG_KEY", false)
        if (flag){
            super.onBackPressed()
        }
    }
}